@@ -1,4 +1,4 @@
-/* $Id: address.c,v 1.47 2007/06/06 02:45:09 rikster5 Exp $ */
+/* $Id: address.c,v 1.50 2007-10-19 17:12:41 rikster5 Exp $ */
 
 /*******************************************************************************
  * address.c
@@ -61,7 +61,7 @@ int address_compare(const void *v1, const void *v2)
 {
    AddressList **al1, **al2;
    struct Address *a1, *a2;
-   char *str1, *str2;
+   char *str1 = NULL, *str2 = NULL;
    int last_cmp1, last_cmp2;
    int sort_idx[4];
    int i, j, r;
@@ -151,8 +151,14 @@ int address_compare(const void *v1, const void *v2)
       return 0;
 
    } else if ((glob_sort_rule & SORT_JAPANESE) && !(glob_sort_rule & SORT_JOS)){
+      /* Japanese sorting has not been updated to fix Bug 1814 because no test
+       * platform is available for Western programmers maintaining Jpilot. */
       int sort1, sort2, sort3;
       char *tmp_p1, *tmp_p2, *tmp_p3;
+      sort1 = sort_idx[1];
+      sort2 = sort_idx[2];
+      sort3 = sort_idx[3];
+
       if (a1->entry[sort1] || a1->entry[sort2]) {
 	 if (a1->entry[sort1] && a1->entry[sort2]) {
 	    if (!(tmp_p1 = strchr(a1->entry[sort1],'\1'))) tmp_p1=a1->entry[sort1]+1;
@@ -239,6 +245,9 @@ int address_compare(const void *v1, const void *v2)
       }
       return i;
    }
+
+   /* Should never be reached.  Assume records are unsortable if reached */
+   return 0;
 }
 
 /*
