@@ -1,4 +1,4 @@
-/* $Id: alarms.c,v 1.38 2006/04/17 05:31:37 rikster5 Exp $ */
+/* $Id: alarms.c,v 1.42 2007-10-10 17:07:33 rikster5 Exp $ */
 
 /*******************************************************************************
  * alarms.c
@@ -24,7 +24,12 @@
  * The PalmOS datebook will alarm on private records even when they are hidden
  * and they show up on the screen.  Right, or wrong, who knows.
  * I will do the same.
+ *
+ * Throughout the code, event_time is the time of the event
+ * alarm_time is the event_time - advance
+ * remind_time is the time a window is to be popped up (it may be postponed)
  */
+
 #include "config.h"
 #include <gtk/gtk.h>
 #include <time.h>
@@ -44,19 +49,17 @@
 /* Every call takes CPU time(not much), so you may want it to be greater */
 #define ALARM_INTERVAL 10
 
+/* Constants used in converting to seconds */
+#define MIN_IN_SECS 60
+#define HR_IN_SECS  3600
+#define DAY_IN_SECS 86400
 
 #define PREV_ALARM_MASK 1
 #define NEXT_ALARM_MASK 2
 
+/* Uncomment for verbose debugging of the alarm code */
 /* #define ALARMS_DEBUG */
 
-
-/*
- * Throughout the code, event_time is the time of the event
- * alarm_time is the event_time - advance
- * remind_time is the time a window is to be popped up (it may be postponed)
- */
-
 typedef enum {
    ALARM_NONE = 0,
    ALARM_NEW,
@@ -97,12 +100,9 @@ void alarms_add_to_list(unsigned int unique_id,
 			time_t alarm_advance);
 int alarms_find_next(struct tm *date1, struct tm *date2, int soonest_only);
 
-/*
- * Alarm GUI
- */
-/*
- * Start of Dialog window code
- */
+/* Alarm GUI */
+
+/* Start of Dialog window code */
 static void cb_dialog_button(GtkWidget *widget,
 			     gpointer   data)
 {
@@ -138,10 +138,10 @@ static gboolean cb_destroy_dialog(GtkWidget *widget)
       set_pref(PREF_REMIND_IN, remind, NULL, TRUE);
       if (GTK_TOGGLE_BUTTON(Pdata->radio1)->active) {
 	 set_pref(PREF_REMIND_UNITS, 0, NULL, TRUE);
-	 remind *= 60;
+	 remind *= MIN_IN_SECS;
       } else {
 	 set_pref(PREF_REMIND_UNITS, 1, NULL, TRUE);
-	 remind *= 3600;
+	 remind *= HR_IN_SECS;
       }
       time(&ltime);
       localtime(&ltime);
@@ -180,7 +180,7 @@ int dialog_alarm(char *title, char *reason,
 #endif
 
    /* Prevent alarms from going crazy and using all resources */
-   if (total_alarm_windows>20) {
+   if (total_alarm_windows > 20) {
       return EXIT_FAILURE;
    }
    total_alarm_windows++;
@@ -242,11 +242,11 @@ int dialog_alarm(char *title, char *reason,
    remind_entry = gtk_spin_button_new_with_range(0, 59, 1);
 #else
    {
-	   GtkAdjustment *spinner_adj;
+      GtkAdjustment *spinner_adj;
 
-	   spinner_adj = (GtkAdjustment *) gtk_adjustment_new(50.0, 0.0, 59.0,
-		   1.0, 5.0, 5.0);
-	   remind_entry = gtk_spin_button_new(spinner_adj, 1.0, 0);
+      spinner_adj = (GtkAdjustment *) gtk_adjustment_new(50.0, 0.0, 59.0,
+                                                          1.0, 5.0, 5.0);
+      remind_entry = gtk_spin_button_new(spinner_adj, 1.0, 0);
    }
 #endif
    gtk_box_pack_start(GTK_BOX(hbox1), remind_entry, FALSE, TRUE, 5);
@@ -309,9 +309,7 @@ int dialog_alarm(char *title, char *reason,
 
    return EXIT_SUCCESS;
 }
-/*
- * End Alarm GUI
- */
+/* End Alarm GUI */
 
 time_t mktime_dst_adj(struct tm *tm)
 {
@@ -354,18 +352,8 @@ const char *print_type(AlarmType type)
       return "? ALARM_UNKNOWN";
    }
 }
-#else
-inline const char *print_date(const time_t t1)
-{
-   return "";
-}
-inline const char *print_type(AlarmType type)
-{
-   return "";
-}
 #endif
 
-
 void alarms_add_to_list(unsigned int unique_id,
 			AlarmType type,
 			time_t event_time,
@@ -499,7 +487,8 @@ void alarms_write_file(void)
    }
 }
 
-/* This attempts to make the command safe.
+/*
+ * This attempts to make the command safe.
  * I'm sure I'm missing things.
  */
 void make_command_safe(char *command)
@@ -517,9 +506,10 @@ void make_command_safe(char *command)
 }
 
 /*
- * Pop up window.
- * Do alarm setting (play sound, or whatever).
- * if user postpones then put in postponed alarm list.
+ * Process an alarm occurrence
+ *   Pop up alarm window.
+ *   Do alarm setting (play sound, or whatever).
+ *   if user postpones then put in postponed alarm list.
  */
 int alarms_do_one(struct Appointment *appt,
 		  unsigned long unique_id,
@@ -651,7 +641,6 @@ int alarms_do_one(struct Appointment *appt,
    return EXIT_SUCCESS;
 }
 
-
 /*
  * See if next_alarm is due in less than ALARM_INTERVAL/2 secs.
  * If it is, then do_alarm and find_next_alarm.
@@ -659,7 +648,7 @@ int alarms_do_one(struct Appointment *appt,
 gint cb_timer_alarms(gpointer data)
 {
    struct jp_alarms *temp_alarm, *ta_next;
-   AppointmentList *a_list;
+   AppointmentList *alm_list;
    AppointmentList *temp_al;
    static int first=0;
    time_t t, diff;
@@ -667,7 +656,7 @@ gint cb_timer_alarms(gpointer data)
    struct tm *Ptm;
    struct tm copy_tm;
 
-   a_list=NULL;
+   alm_list=NULL;
 
    if (!first) {
       alarms_write_file();
@@ -684,8 +673,8 @@ gint cb_timer_alarms(gpointer data)
 	    continue;
 	 }
       }
-      if (a_list==NULL) {
-	 get_days_appointments2(&a_list, NULL, 0, 0, 1, NULL);
+      if (alm_list==NULL) {
+	 get_days_appointments2(&alm_list, NULL, 0, 0, 1, NULL);
       }
 #ifdef ALARMS_DEBUG
       printf("unique_id=%d\n", temp_alarm->unique_id);
@@ -693,7 +682,7 @@ gint cb_timer_alarms(gpointer data)
       printf("event_time=%s\n", print_date(temp_alarm->event_time));
       printf("alarm_advance=%ld\n", temp_alarm->alarm_advance);
 #endif
-      for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
+      for (temp_al = alm_list; temp_al; temp_al=temp_al->next) {
 	 if (temp_al->mappt.unique_id == temp_alarm->unique_id) {
 #ifdef ALARMS_DEBUG
 	    printf("%s\n", temp_al->mappt.appt.description);
@@ -705,7 +694,7 @@ gint cb_timer_alarms(gpointer data)
 	    break;
 	 }
       }
-      /* Be careful, this modifies the list we are parsing and
+      /* CAUTION, this modifies the list we are parsing and
        removes the current node */
       alarms_remove_from_to_list(temp_al->mappt.unique_id);
    }
@@ -713,11 +702,11 @@ gint cb_timer_alarms(gpointer data)
    if (next_alarm) {
       diff = next_alarm->event_time - t - next_alarm->alarm_advance;
       if (diff <= ALARM_INTERVAL/2) {
-	 if (a_list==NULL) {
-	    get_days_appointments2(&a_list, NULL, 0, 0, 1, NULL);
+	 if (alm_list==NULL) {
+	    get_days_appointments2(&alm_list, NULL, 0, 0, 1, NULL);
 	 }
 	 for (temp_alarm=next_alarm; temp_alarm; temp_alarm=ta_next) {
-	    for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
+	    for (temp_al = alm_list; temp_al; temp_al=temp_al->next) {
 	       if (temp_al->mappt.unique_id == temp_alarm->unique_id) {
 #ifdef ALARMS_DEBUG
 		  printf("** next unique_id=%d\n", temp_alarm->unique_id);
@@ -747,8 +736,8 @@ gint cb_timer_alarms(gpointer data)
 	 alarms_find_next(&copy_tm, &copy_tm, TRUE);
       }
    }
-   if (a_list) {
-      free_AppointmentList(&a_list);
+   if (alm_list) {
+      free_AppointmentList(&alm_list);
    }
 
    return TRUE;
@@ -758,18 +747,15 @@ gint cb_timer_alarms(gpointer data)
  * This routine takes time (t) and either advances t to the next
  * occurrence of a repeating appointment, or the previous occurrence
  *
- * a is the appointment passed in
- * t is an in/out param.
- * fdom is first day of month (Sunday being 0)
- * ndim is the number of days in the month
- * forward_or_backward should be -1 for backward or 1 for forward.
+ * appt is the appointment passed in
+ * t is an in/out parameter
+ * forward_or_backward should be 1 for forward or -1 for backward
  */
 int forward_backward_in_appt_time(const struct Appointment *appt,
 				  struct tm *t,
-				  int fdom, int ndim,
 				  int forward_or_backward)
 {
-   int count, dow, freq;
+   int count, dow, freq, fdom, ndim;
 
    freq = appt->repeatFrequency;
 
@@ -777,26 +763,30 @@ int forward_backward_in_appt_time(const struct Appointment *appt,
    if (forward_or_backward==1) {
       switch (appt->repeatType) {
        case repeatNone:
+#ifdef ALARMS_DEBUG
+         printf("fbiat: repeatNone encountered.  This should never happen!\n");
+#endif
 	 break;
        case repeatDaily:
+         add_days_to_date(t, freq);
 	 break;
        case repeatWeekly:
 	 for (count=0, dow=t->tm_wday; count<14; count++) {
 	    add_days_to_date(t, 1);
 #ifdef ALARMS_DEBUG
-	    printf("fpn: weekly forward t.tm_wday=%d, freq=%d\n", t->tm_wday, freq);
+	    printf("fbiat: weekly forward t.tm_wday=%d, freq=%d\n", t->tm_wday, freq);
 #endif
 	    dow++;
 	    if (dow==7) {
 #ifdef ALARMS_DEBUG
-	       printf("fpn: dow==7\n");
+	       printf("fbiat: dow==7\n");
 #endif
 	       add_days_to_date(t, (freq-1)*7);
 	       dow=0;
 	    }
 	    if (appt->repeatDays[dow]) {
 #ifdef ALARMS_DEBUG
-	       printf("fpn: repeatDay[dow] dow=%d\n", dow);
+	       printf("fbiat: repeatDay[dow] dow=%d\n", dow);
 #endif
 	       break;
 	    }
@@ -818,33 +808,39 @@ int forward_backward_in_appt_time(const struct Appointment *appt,
 	 t->tm_mday=appt->begin.tm_mday;
 	 add_years_to_date(t, freq);
 	 break;
-      }/*switch */
+      } /* switch on repeatType */
+
       return EXIT_SUCCESS;
    }
+
    /* Go back in time */
    if (forward_or_backward==-1) {
       switch (appt->repeatType) {
        case repeatNone:
+#ifdef ALARMS_DEBUG
+         printf("fbiat: repeatNone encountered.  This should never happen!\n");
+#endif
 	 break;
        case repeatDaily:
+         sub_days_from_date(t, freq);
 	 break;
        case repeatWeekly:
 	 for (count=0, dow=t->tm_wday; count<14; count++) {
 	    sub_days_from_date(t, 1);
 #ifdef ALARMS_DEBUG
-	    printf("fpn: weekly backward t.tm_wday=%d, freq=%d\n", t->tm_wday, freq);
+	    printf("fbiat: weekly backward t.tm_wday=%d, freq=%d\n", t->tm_wday, freq);
 #endif
 	    dow--;
 	    if (dow==-1) {
 #ifdef ALARMS_DEBUG
-	       printf("fpn: dow==-1\n");
+	       printf("fbiat: dow==-1\n");
 #endif
 	       sub_days_from_date(t, (freq-1)*7);
 	       dow=6;
 	    }
 	    if (appt->repeatDays[dow]) {
 #ifdef ALARMS_DEBUG
-	       printf("fpn: repeatDay[dow] dow=%d\n", dow);
+	       printf("fbiat: repeatDay[dow] dow=%d\n", dow);
 #endif
 	       break;
 	    }
@@ -866,13 +862,28 @@ int forward_backward_in_appt_time(const struct Appointment *appt,
 	 t->tm_mday=appt->begin.tm_mday;
 	 sub_years_from_date(t, freq);
 	 break;
-      }/*switch */
+      } /* switch on repeatType */
    }
+
    return EXIT_SUCCESS;
 }
 
+/*
+ * Search forwards and backwards in time to find alarms which bracket 
+ * date1 and date2.
+ *   For non-repeating appointments no searching is necessary.  
+ *   The appt is either in the range or it is not.
+ *   The algorithm for searching is time consuming.  To improve performance 
+ *   this subroutine seeks to seed the search with a close guess for the 
+ *   correct time before launching the search.
+ *
+ *   Math is explicitly done with integers so that divisions which might produce
+ *   a float as a result will instead produce a truncated result.  
+ *   Alternatively the C math functions such as floor could be used but there 
+ *   seems little point in invoking such overhead.
+ */
 static int find_prev_next(struct Appointment *appt,
-			  int adv,
+			  time_t adv,
 			  struct tm *date1,
 			  struct tm *date2,
 			  struct tm *tm_prev,
@@ -881,25 +892,23 @@ static int find_prev_next(struct Appointment *appt,
 			  int *next_found)
 {
    struct tm t;
-   struct tm *Pnow;
-   time_t t_temp;
    time_t t1, t2;
    time_t t_begin, t_end;
    time_t t_alarm;
-   time_t t_past;
-   time_t t_future;
-   time_t t_interval;
+   time_t t_offset;
+   time_t t_temp;
    int forward, backward;
    int offset;
    int freq;
-   int found;
-   int count;
-   int i;
-   int safety_counter;
+   int date1_days, begin_days;
    int fdom, ndim;
-   long fdow;
-   int days, begin_days;
+   int found, count, i;
+   int safety_counter;
    int found_exception;
+   int kill_update_next;
+#ifdef ALARMS_DEBUG
+   char str[100];
+#endif
 
 #ifdef ALARMS_DEBUG
    printf("fpn: entered find_previous_next\n");
@@ -913,6 +922,7 @@ static int find_prev_next(struct Appointment *appt,
    memset(tm_prev, 0, sizeof(*tm_prev));
    memset(tm_next, 0, sizeof(*tm_next));
 
+   /* Initialize search time with appt start time */
    memset(&t, 0, sizeof(t));
    t.tm_year=appt->begin.tm_year;
    t.tm_mon=appt->begin.tm_mon;
@@ -920,18 +930,14 @@ static int find_prev_next(struct Appointment *appt,
    t.tm_hour=appt->begin.tm_hour;
    t.tm_min=appt->begin.tm_min;
    t.tm_isdst=-1;
-
    mktime(&t);
 #ifdef ALARMS_DEBUG
-	{
-	   char str[100];
-	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
-	   printf("fpn: debug=%s\n", str);
-	}
+   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
+   printf("fpn: appt_start=%s\n", str);
 #endif
-   freq = 0;
-   switch (appt->repeatType) {
-    case repeatNone:
+
+   /* Handle non-repeating appointments */        
+   if (appt->repeatType == repeatNone) {
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatNone\n");
 #endif
@@ -949,215 +955,149 @@ static int find_prev_next(struct Appointment *appt,
 	 printf("fpn: next_found none\n");
 #endif
       }
-      forward=backward=0;
+      return EXIT_SUCCESS;
+   }
+
+   /* Optimize initial start position of search */ 
+   switch (appt->repeatType) {
+    case repeatNone:
+      /* Already handled.  Here only to shut up compiler warnings */
       break;
     case repeatDaily:
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatDaily\n");
 #endif
       freq = appt->repeatFrequency;
-      t_interval = appt->repeatFrequency * 86400;
-      if (t_interval==0) t_interval=1;
+      t_offset = freq * DAY_IN_SECS;
       t_alarm = mktime_dst_adj(&t);
-      if ((t2 + adv) > t_alarm) {
-	 t_past = ((t2 + adv - t_alarm) / t_interval) *t_interval + t_alarm;
-	 t_future = (((t2 + adv - t_alarm) / t_interval) + 1) *t_interval + t_alarm;
-	 *prev_found=*next_found=1;
-      } else {
-	 t_future = t_alarm;
-	 *next_found=1;
-      }
-      Pnow = localtime(&t_past);
-      memcpy(tm_prev, Pnow, sizeof(struct tm));
-      Pnow = localtime(&t_future);
-      memcpy(tm_next, Pnow, sizeof(struct tm));
-      forward=backward=0;
-      /* since the code above disregarded DST in the above calcs,
-       * we have to try to correct for it here */
-      if (tm_prev->tm_hour<appt->begin.tm_hour) {
-	 t_past+=3600; /* 1 hour for dst */
-	 Pnow = localtime(&t_past);
-	 memcpy(tm_prev, Pnow, sizeof(struct tm));
-      }
-      if (tm_prev->tm_hour>appt->begin.tm_hour) {
-	 t_past-=3600; /* 1 hour for dst */
-	 Pnow = localtime(&t_past);
-	 memcpy(tm_prev, Pnow, sizeof(struct tm));
-      }
-      if (tm_next->tm_hour<appt->begin.tm_hour) {
-	 t_future+=3600; /* 1 hour for dst */
-	 Pnow = localtime(&t_future);
-	 memcpy(tm_next, Pnow, sizeof(struct tm));
-      }
-      if (tm_next->tm_hour>appt->begin.tm_hour) {
-	 t_future-=3600; /* 1 hour for dst */
-	 Pnow = localtime(&t_future);
-	 memcpy(tm_next, Pnow, sizeof(struct tm));
-      }
-      if (t_future<t1) {
-  	 *next_found=0;
-	 forward=1;
-      }
+      /* Jump to closest current date if appt. started in the past */
+      if (t1 - adv > t_alarm)
+      {
+	 t_alarm = ((((t1+adv)-t_alarm) / t_offset) * t_offset) + t_alarm;
+         memcpy(&t, localtime(&t_alarm), sizeof(struct tm));
 #ifdef ALARMS_DEBUG
-	{
-	   char str[100];
-	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", tm_prev);
-	   printf("fpn: daily tm_prev=%s\n", str);
-	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", tm_next);
-	   printf("fpn: daily tm_next=%s\n", str);
-	}
+         strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
+         printf("fpn: initial daily=%s\n", str);
 #endif
+      }
       break;
     case repeatWeekly:
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatWeekly\n");
 #endif
       freq = appt->repeatFrequency;
-      /* Check for future start date */
       begin_days = dateToDays(&(appt->begin));
-      days = dateToDays(&t);
-      if (days > begin_days) {
-	 /* appt started in past, get to current date sooner */
-	 t.tm_year=date2->tm_year;
-	 t.tm_mon=date2->tm_mon;
-	 t.tm_mday=date2->tm_mday;
-      }
-      t.tm_isdst=-1;
-      mktime(&t);
-      begin_days = dateToDays(&(appt->begin));
-      days = dateToDays(&t);
+      date1_days = dateToDays(date1);
+      /* Jump to closest current date if appt. started in the past */
+      if (date1_days > begin_days) {
 #ifdef ALARMS_DEBUG
-      printf("fpn: begin_days %d days %d\n", begin_days, days);
-      printf("fpn: t.tm_wday %d appt->begin.tm_wday %d\n", t.tm_wday, appt->begin.tm_wday);
+         printf("fpn: begin_days %d date1_days %d\n", begin_days, date1_days);
+         printf("fpn: date1->tm_wday %d appt->begin.tm_wday %d\n", date1->tm_wday, appt->begin.tm_wday);
+#endif
+         /* Jump by appropriate number of weeks */
+         offset = date1_days - begin_days;
+         offset = (offset/(freq*7))*(freq*7);
+#ifdef ALARMS_DEBUG
+         printf("fpn: offset %d\n", offset);
 #endif
-      get_pref(PREF_FDOW, &fdow, NULL);
 
-      /* Offset is how many weeks we are off of an iteration */
-      offset = ((int)((days - t.tm_wday) - (begin_days - appt->begin.tm_wday))/7)%freq;
-#ifdef ALARMS_DEBUG
-      printf("fpn: offset %d\n", offset);
-#endif
-      if (offset > 0) {
-	 sub_days_from_date(&t, offset*7);
-      } else {
-	 add_days_to_date(&t, offset*-7);
+         add_days_to_date(&t, offset);
       }
+
+      /* Within the week find which day is a repeat */
       found=0;
       for (count=0, i=t.tm_wday; i>=0; i--, count++) {
-	 if (appt->repeatDays[i]) {
-	    sub_days_from_date(&t, count);
-	    found=1;
-#ifdef ALARMS_DEBUG
-	      {
-		 char str[100];
-		 strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
-		 printf("fpn: initial weekly=%s\n", str);
-	      }
-#endif
-	    break;
-	 }
+         if (appt->repeatDays[i]) {
+            sub_days_from_date(&t, count);
+            found=1;
+            break;
+         }
       }
       if (!found) {
-	 for (count=0, i=t.tm_wday; i<7; i++, count++) {
-	    if (appt->repeatDays[i]) {
-	       add_days_to_date(&t, count);
-	       found=1;
-	       break;
-	    }
-	 }
+         for (count=0, i=t.tm_wday; i<7; i++, count++) {
+            if (appt->repeatDays[i]) {
+               add_days_to_date(&t, count);
+               found=1;
+               break;
+            }
+         }
       }
+#ifdef ALARMS_DEBUG
+         strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
+         printf("fpn: initial weekly=%s\n", str);
+#endif
       break;
     case repeatMonthlyByDay:
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatMonthlyByDay\n");
 #endif
-      /* Check for future start date */
-      begin_days = dateToDays(&(appt->begin));
-      days = dateToDays(&t);
-      if (days > begin_days) {
-	 /* appt started in past, get to current date sooner */
-	 t.tm_mon=date2->tm_mon;
-	 t.tm_year=date2->tm_year;
-      }
-      freq = appt->repeatFrequency;
-#ifdef ALARMS_DEBUG
-      printf("fpn: freq=%d\n", freq);
-#endif
-      offset = ((t.tm_year - appt->begin.tm_year)*12 +
-		(t.tm_mon - appt->begin.tm_mon))%(appt->repeatFrequency);
-      /* This will adjust for leap year, and exceeding the end of the month */
-      if (((t.tm_year - appt->begin.tm_year)*12 + (t.tm_mon - appt->begin.tm_mon)) < 0) {
+      /* Jump to closest current date if appt. started in the past */
+      if ((date1->tm_year > appt->begin.tm_year) ||
+          (date1->tm_mon  > appt->begin.tm_mon)) {
+         /* First, adjust month */
+         freq = appt->repeatFrequency;
+         offset = ((date1->tm_year - appt->begin.tm_year)*12) -
+                    appt->begin.tm_mon +
+                    date1->tm_mon;
+         offset = (offset/freq)*freq;
 	 add_months_to_date(&t, offset);
-      } else {
-	 sub_months_from_date(&t, offset);
-      }
-      get_month_info(t.tm_mon, 1, t.tm_year, &fdom, &ndim);
-      t.tm_mday=((appt->repeatDay+7-fdom)%7) - ((appt->repeatDay)%7) + appt->repeatDay + 1;
+
+         /* Second, adjust to correct day in new month */
+	 get_month_info(t.tm_mon, 1, t.tm_year, &fdom, &ndim);
+	 t.tm_mday=((appt->repeatDay+7-fdom)%7) - ((appt->repeatDay)%7) + appt->repeatDay + 1;
 #ifdef ALARMS_DEBUG
-      printf("fpn: %02d/01/%02d, fdom=%d\n", t.tm_mon+1, t.tm_year+1900, fdom);
-      printf("fpn: mday = %d\n", t.tm_mday);
+         printf("fpn: months offset = %d\n", offset);
+         printf("fpn: %02d/01/%02d, fdom=%d\n", t.tm_mon+1, t.tm_year+1900, fdom);
+         printf("fpn: mday = %d\n", t.tm_mday);
 #endif
-      if (t.tm_mday > ndim) {
-	 t.tm_mday -= 7;
-      }
+	 if (t.tm_mday > ndim-1) {
+	    t.tm_mday -= 7;
+	 }
 #ifdef ALARMS_DEBUG
-	{
-	   char str[100];
-	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
-	   printf("fpn: initial monthly by day=%s\n", str);
-	}
+         strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
+         printf("fpn: initial monthly by day=%s\n", str);
 #endif
+      }
       break;
     case repeatMonthlyByDate:
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatMonthlyByDate\n");
 #endif
-      /* Check for future start date */
-      begin_days = dateToDays(&(appt->begin));
-      days = dateToDays(&t);
-      if (days > begin_days) {
-	 /* appt started in past, get to current date sooner */
-	 t.tm_mon=date2->tm_mon;
-	 t.tm_year=date2->tm_year;
-      }
-
-      freq = appt->repeatFrequency;
-      offset = ((t.tm_year - appt->begin.tm_year)*12 +
-		(t.tm_mon - appt->begin.tm_mon))%(appt->repeatFrequency);
-      /* This will adjust for leap year, and exceeding the end of the month */
-      if (((t.tm_year - appt->begin.tm_year)*12 + (t.tm_mon - appt->begin.tm_mon)) < 0) {
+      /* Jump to closest current date if appt. started in the past */
+      if ((date1->tm_year > appt->begin.tm_year) ||
+          (date1->tm_mon  > appt->begin.tm_mon)) {
+         freq = appt->repeatFrequency;
+         offset = ((date1->tm_year - appt->begin.tm_year)*12) -
+                    appt->begin.tm_mon +
+                    date1->tm_mon;
+         offset = (offset/freq)*freq;
+#ifdef ALARMS_DEBUG
+         printf("fpn: months offset = %d\n", offset);
+#endif
 	 add_months_to_date(&t, offset);
-      } else {
-	 sub_months_from_date(&t, offset);
       }
       break;
     case repeatYearly:
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatYearly\n");
 #endif
-      begin_days = dateToDays(&(appt->begin));
-      days = dateToDays(&t);
-      if (days > begin_days) {
-	 /* appt started in past, get to current date sooner */
-	 t.tm_year=date2->tm_year;
-      }
-      freq = appt->repeatFrequency;
-      offset = (t.tm_year - appt->begin.tm_year)%(appt->repeatFrequency);
+      /* Jump to closest current date if appt. started in the past */
+      if (date1->tm_year > appt->begin.tm_year) {
+         freq = appt->repeatFrequency;
+         offset = ((date1->tm_year - appt->begin.tm_year)/freq)*freq;
 #ifdef ALARMS_DEBUG
-      printf("fpn: (%d - %d)%%%d\n", t.tm_year,appt->begin.tm_year,appt->repeatFrequency);
-      printf("fpn: *** years offset = %d\n", offset);
+         printf("fpn: (%d - %d)%%%d\n", date1->tm_year, appt->begin.tm_year, freq);
+         printf("fpn: years offset = %d\n", offset);
 #endif
-      /* This will adjust for leap year, and exceeding the end of the month */
-      if ((t.tm_year - appt->begin.tm_year) < 0) {
 	 add_years_to_date(&t, offset);
-      } else {
-	 sub_years_from_date(&t, offset);
       }
       break;
-   }
 
+   } /* end switch on repeatType */
+
+   /* Search forwards/backwards through time for alarms */
    safety_counter=0;
-   while(forward || backward) {
+   while (forward || backward) {
       safety_counter++;
       if (safety_counter > 3000) {
 	 jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "find_prev_next(): %s\n", _("infinite loop, breaking\n"));
@@ -1166,17 +1106,15 @@ static int find_prev_next(struct Appointment *appt,
 	 }
 	 break;
       }
+
+      kill_update_next = 0;
       t_temp = mktime_dst_adj(&t);
-      t.tm_isdst=-1;
-      t_temp = mktime(&t);
 #ifdef ALARMS_DEBUG
-	{
-	   char str[100];
-	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
-	   printf("fpn: trying with=%s\n", str);
-	}
+      strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
+      printf("fpn: trying with=%s\n", str);
 #endif
-      /* Check for exceptions */
+
+      /* Check for exceptions in repeat appointments */
       found_exception=0;
       for (i=0; i<appt->exceptions; i++) {
 	 if ((t.tm_mday==appt->exception[i].tm_mday) &&
@@ -1189,40 +1127,46 @@ static int find_prev_next(struct Appointment *appt,
       }
       if (found_exception) {
 	 if (forward) {
-	    forward_backward_in_appt_time(appt, &t, fdom, ndim, 1);
+	    forward_backward_in_appt_time(appt, &t, 1);
 	    continue;
 	 }
 	 if (backward) {
-	    forward_backward_in_appt_time(appt, &t, fdom, ndim, -1);
+	    forward_backward_in_appt_time(appt, &t, -1);
 	    continue;
 	 }
       }
-      /* See that we aren't before the begin date */
+
+      /* Check that proposed alarm is not before the appt begin date */
       t_begin = mktime_dst_adj(&(appt->begin));
-      if (t_temp < t_begin - adv) {
+      if (t_temp < t_begin) {
 #ifdef ALARMS_DEBUG
 	 printf("fpn: before begin date\n");
 #endif
 	 backward=0;
+         kill_update_next = 1;
       }
-      /* If the appointment has an end date, see that we are not past it */
+      /* Check that proposed alarm is not past appt end date */
       if (!(appt->repeatForever)) {
 	 t_end = mktime_dst_adj(&(appt->repeatEnd));
-	 if (t_temp > t_end) {
+	 if (t_temp >= t_end) {
 #ifdef ALARMS_DEBUG
 	    printf("fpn: after end date\n");
 #endif
 	    forward=0;
 	 }
       }
+
+      /* Check if proposed alarm falls within the desired window [t1,t2] */
       t_temp-=adv;
       if (t_temp >= t2) {
-	 memcpy(tm_next, &t, sizeof(t));
-	 *next_found=1;
-	 forward=0;
+         if (!kill_update_next) {
+            memcpy(tm_next, &t, sizeof(t));
+            *next_found=1;
+            forward=0;
 #ifdef ALARMS_DEBUG
-	 printf("fpn: next found\n");
+            printf("fpn: next found\n");
 #endif
+         }
       } else {
 	 memcpy(tm_prev, &t, sizeof(t));
 	 *prev_found=1;
@@ -1231,27 +1175,32 @@ static int find_prev_next(struct Appointment *appt,
 	 printf("fpn: prev_found\n");
 #endif
       }
+
+      /* Change &t to the next/previous occurrence of the appointment 
+       * and try search again */
       if (forward) {
-	 forward_backward_in_appt_time(appt, &t, fdom, ndim, 1);
+	 forward_backward_in_appt_time(appt, &t, 1);
 	 continue;
       }
       if (backward) {
-	 forward_backward_in_appt_time(appt, &t, fdom, ndim, -1);
+	 forward_backward_in_appt_time(appt, &t, -1);
 	 continue;
       }
-   }
+
+   } /* end of while loop going forward/backward */
+
    return EXIT_SUCCESS;
 }
 
 
 /*
  * Find the next appointment alarm
- *  if soonest_only then return the next alarm, else return all alarms
- *  that occur between the 2 dates.
+ *  if soonest_only then return the next alarm, 
+ *  else return all alarms that occur between the two dates.
  */
 int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 {
-   AppointmentList *a_list;
+   AppointmentList *alm_list;
    AppointmentList *temp_al;
    struct jp_alarms *ta;
 
@@ -1262,8 +1211,6 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    time_t t_begin, t_end;
    time_t t_prev;
    time_t t_future;
-   time_t t_interval;
-   time_t t_soonest;
    struct tm *tm_temp;
    struct tm date1, date2;
    struct tm tm_prev, tm_next;
@@ -1308,17 +1255,16 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    t2=mktime_dst_adj(&date2);
 
 #ifdef ALARMS_DEBUG
-     {
-	char str[100];
-	struct tm *Pnow;
-	strftime(str, sizeof(str), "%B %d, %Y %H:%M", &date1);
-	printf("date1=%s\n", str);
-	strftime(str, sizeof(str), "%B %d, %Y %H:%M", &date2);
-	printf("date2=%s\n", str);
-	Pnow = localtime(&t1);
-	strftime(str, sizeof(str), "%B %d, %Y %H:%M", Pnow);
-	printf("^date1=%s\n", str);
-     }
+   char str[100];
+   struct tm *Pnow;
+
+   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &date1);
+   printf("date1=%s\n", str);
+   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &date2);
+   printf("date2=%s\n", str);
+   Pnow = localtime(&t1);
+   strftime(str, sizeof(str), "%B %d, %Y %H:%M", Pnow);
+   printf("[Now]=%s\n", str);
 #endif
 
    if (!soonest_only) {
@@ -1327,77 +1273,73 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       free_alarms_list(NEXT_ALARM_MASK);
    }
 
-   a_list=NULL;
-   get_days_appointments2(&a_list, NULL, 0, 0, 1, NULL);
+   alm_list=NULL;
+   get_days_appointments2(&alm_list, NULL, 0, 0, 1, NULL);
 
-   t_soonest=0;
-
-   for (temp_al=a_list; temp_al; temp_al=temp_al->next) {
-      /*
-       * No alarm, skip
-       */
+   for (temp_al=alm_list; temp_al; temp_al=temp_al->next) {
+      /* No alarm, skip */
       if (!temp_al->mappt.appt.alarm) {
 	 continue;
       }
 #ifdef ALARMS_DEBUG
       printf("\n[%s]\n", temp_al->mappt.appt.description);
 #endif
-      /*
-       * See if the appointment starts before date1
-       * and is a non repeating appointment
-       */
+      /* Check for ordinary non-repeating appt starting before date1 */
       if (temp_al->mappt.appt.repeatType == repeatNone) {
 	 t_alarm = mktime_dst_adj(&(temp_al->mappt.appt.begin));
 	 if (t_alarm < t1) {
 #ifdef ALARMS_DEBUG
-	    printf("non repeat before t1, t_alarm<t1, %ld<%ld\n",t_alarm,t1);
+	    printf("afn: non repeat before t1, t_alarm<t1, %ld<%ld\n",t_alarm,t1);
 #endif
 	    continue;
 	 }
       }
 
-      /* If the appointment has an end date, see that we are not past it */
+      /* Check that we are not past any appointment end date */
       if (!(temp_al->mappt.appt.repeatForever)) {
 	 t_end = mktime_dst_adj(&(temp_al->mappt.appt.repeatEnd));
 	 /* We need to add 24 hours to the end date to make it inclusive */
-	 t_end += 86400;
+	 t_end += DAY_IN_SECS;
 	 t_begin = mktime_dst_adj(&(temp_al->mappt.appt.begin));
 	 if (t_end < t2) {
 #ifdef ALARMS_DEBUG
-	    printf("past end date\n");
+	    printf("afn: past end date\n");
 #endif
 	    continue;
 	 }
       }
 
+      /* Calculate the alarm advance in seconds */
+      adv = 0;
+      switch (temp_al->mappt.appt.advanceUnits) {
+       case advMinutes:
+	 adv = temp_al->mappt.appt.advance*MIN_IN_SECS;
+         break;
+       case advHours:
+	 adv = temp_al->mappt.appt.advance*HR_IN_SECS;
+         break;
+       case advDays:
+	 adv = temp_al->mappt.appt.advance*DAY_IN_SECS;
+         break;
+      }
+
 #ifdef ALARMS_DEBUG
       printf("alarm advance %d ", temp_al->mappt.appt.advance);
-#endif
-      adv = 0;
-      if (temp_al->mappt.appt.advanceUnits == advMinutes) {
-#ifdef ALARMS_DEBUG
+      switch (temp_al->mappt.appt.advanceUnits) {
+       case advMinutes:
 	 printf("minutes\n");
-#endif
-	 adv = temp_al->mappt.appt.advance*60;
-      }
-      if (temp_al->mappt.appt.advanceUnits == advHours) {
-#ifdef ALARMS_DEBUG
+         break;
+       case advHours:
 	 printf("hours\n");
-#endif
-	 adv = temp_al->mappt.appt.advance*3600;
-      }
-      if (temp_al->mappt.appt.advanceUnits == advDays) {
-#ifdef ALARMS_DEBUG
+         break;
+       case advDays:
 	 printf("days\n");
-#endif
-	 adv = temp_al->mappt.appt.advance*86400;
+         break;
       }
-
-#ifdef ALARMS_DEBUG
       printf("adv=%ld\n", adv);
 #endif
 
-      t_prev=t_future=t_interval=0;
+      t_prev=t_future=0;
       prev_found=next_found=0;
 
       find_prev_next(&(temp_al->mappt.appt),
@@ -1411,13 +1353,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       t_prev=mktime_dst_adj(&tm_prev);
       t_future=mktime_dst_adj(&tm_next);
 
-#ifdef ALARMS_DEBUG
-      printf("adv = %ld\n", adv);
-#endif
-      /*
-       * Skip the alarms if they are before date1, or after date2
-       */
-
+      /* Skip the alarms if they are before date1 or after date2 */
       if (prev_found) {
 	 if (t_prev - adv < t1) {
 #ifdef ALARMS_DEBUG
@@ -1433,8 +1369,11 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 	 }
       }
       if (next_found) {
+         /* Check that we are not past any appointment end date */
 	 if (!(temp_al->mappt.appt.repeatForever)) {
 	    t_end = mktime_dst_adj(&(temp_al->mappt.appt.repeatEnd));
+            /* We need to add 24 hours to the end date to make it inclusive */
+            t_end += DAY_IN_SECS;
 	    if (t_future > t_end) {
 #ifdef ALARMS_DEBUG
 	       printf("failed future is after t_end\n");
@@ -1448,9 +1387,10 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       printf("t2=       %s\n", print_date(t2));
       printf("t_prev=   %s\n", prev_found ? print_date(t_prev):"None");
       printf("t_future= %s\n", next_found ? print_date(t_future):"None");
-      printf("alarm me= %s\n", next_found ? print_date(t_future):"None");
+      printf("alarm me= %s\n", next_found ? print_date(t_future-adv):"None");
       printf("desc=[%s]\n", temp_al->mappt.appt.description);
 #endif
+
       if (!soonest_only) {
 	 if (prev_found) {
 	    alarms_add_to_list(temp_al->mappt.unique_id, ALARM_MISSED, t_prev, adv);
@@ -1461,9 +1401,9 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 	 if (next_alarm==NULL) {
 	    add_a_next=1;
 	 } else if
-	   (t_future - adv <= next_alarm->event_time - next_alarm->alarm_advance) {
+	   (t_future - adv <= (next_alarm->event_time - next_alarm->alarm_advance)) {
 	      add_a_next=1;
-	      if (t_future - adv < next_alarm->event_time - next_alarm->alarm_advance) {
+	      if (t_future - adv < (next_alarm->event_time - next_alarm->alarm_advance)) {
 #ifdef ALARMS_DEBUG
 		 printf("next alarm=%s\n", print_date(next_alarm->event_time - next_alarm->alarm_advance));
 		 printf("freeing next alarms\n");
@@ -1471,6 +1411,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 		 free_alarms_list(NEXT_ALARM_MASK);
 	      }
 	   }
+
 	 if (add_a_next) {
 #ifdef ALARMS_DEBUG
 	    printf("found a new next\n");
@@ -1487,13 +1428,13 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 	 }
       }
    }
-   free_AppointmentList(&a_list);
+   free_AppointmentList(&alm_list);
 
    return EXIT_SUCCESS;
 }
 
 /*
- * At startup see when rc file was written and find all past-due alarms.
+ * At startup check when rc file was written and find all past-due alarms.
  * Add them to the postponed alarm list with 0 minute reminder.
  * Find next alarm and put it in list
  */
