@@ -1,4 +1,7 @@
-/* datebook.h
+/* $Id: datebook.h,v 1.21 2005/10/24 15:27:48 rikster5 Exp $ */
+
+/*******************************************************************************
+ * datebook.h
  * A module of J-Pilot http://jpilot.org
  * 
  * Copyright (C) 1999-2002 by Judd Montgomery
@@ -15,7 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+ ******************************************************************************/
+
 #ifndef __DATEBOOK_H__
 #define __DATEBOOK_H__
 
@@ -91,12 +95,16 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
 			   int modified, int deleted, int privates,
 			   int *total_records);
 
-/* This funtion removes appointments from the list that obviously will not
+/* This function removes appointments from the list that obviously will not
  * occur in this month.  Skip privates will ignore private records.
  */
 int weed_datebook_list(AppointmentList **al, int mon, int year,
 		       int skip_privates, int *mask);
 
+/* Sorts a list of appointments according to the comparison function given */
+int datebook_sort(AppointmentList **al, 
+                  int (*compare_func)(const void*, const void*));
+
 /* Year is years since 1900 */
 /* Mon is 0-11 */
 /* Day is 1-31 */
@@ -116,8 +124,6 @@ int appointment_on_day_list(int mon, int year, int *mask);
  */
 unsigned int isApptOnDate(struct Appointment *a, struct tm *date);
 
-int dateToSecs(struct tm *tm1);
-
 int dateToDays(struct tm *tm1);
 
 int compareTimesToDay(struct tm *tm1, struct tm *tm2);
