@@ -1,4 +1,4 @@
-/* $Id: plugins.c,v 1.17 2007/06/06 02:45:10 rikster5 Exp $ */
+/* $Id: plugins.c,v 1.19 2007-10-06 15:36:43 rousseau Exp $ */
 
 /*******************************************************************************
  * plugins.c
@@ -81,6 +81,8 @@ int load_plugins_sub1(DIR *dir, char *path, int *number, unsigned char user_only
    struct dirent *dirent;
    char full_name[FILENAME_MAX];
    struct plugin_s temp_plugin, *new_plugin;
+   GList *plugin_names = NULL; /* keep a list of plugins found so far */
+   GList *temp_list = NULL;
 
    count = 0;
    for (i=0; (dirent = readdir(dir)); i++) {
@@ -90,7 +92,8 @@ int load_plugins_sub1(DIR *dir, char *path, int *number, unsigned char user_only
       }
       /* If the filename has either of these extensions then plug it in */
       if ((strcmp(&(dirent->d_name[strlen(dirent->d_name)-3]), ".so")) &&
-	  (strcmp(&(dirent->d_name[strlen(dirent->d_name)-3]), ".sl"))) {
+	  (strcmp(&(dirent->d_name[strlen(dirent->d_name)-3]), ".sl")) &&
+	  (strcmp(&(dirent->d_name[strlen(dirent->d_name)-6]), ".dylib"))) {
 	 continue;
       } else {
 	 jp_logf(JP_LOG_DEBUG, "found plugin %s\n", dirent->d_name);
@@ -103,20 +106,29 @@ int load_plugins_sub1(DIR *dir, char *path, int *number, unsigned char user_only
 	    if (temp_plugin.name) {
 	       jp_logf(JP_LOG_DEBUG, "plugin name is [%s]\n", temp_plugin.name);
 	    }
-	    new_plugin = malloc(sizeof(struct plugin_s));
-	    if (!new_plugin) {
-	       jp_logf(JP_LOG_WARN, "load plugins(): %s\n", _("Out of memory"));
-	       return count;
+	    if (g_list_find_custom(plugin_names, temp_plugin.name, (GCompareFunc)strcmp) == NULL) {
+	       new_plugin = malloc(sizeof(struct plugin_s));
+	       if (!new_plugin) {
+		  jp_logf(JP_LOG_WARN, "load plugins(): %s\n", _("Out of memory"));
+		  return count;
+	       }
+	       memcpy(new_plugin, &temp_plugin, sizeof(struct plugin_s));
+	       plugins = g_list_prepend(plugins, new_plugin);
+	       plugin_names = g_list_prepend(plugin_names, g_strdup(temp_plugin.name));
+	       count++;
+	       (*number)++;
 	    }
-	    memcpy(new_plugin, &temp_plugin, sizeof(struct plugin_s));
-	    plugins = g_list_prepend(plugins, new_plugin);
-	    count++;
-	    (*number)++;
 	 }
       }
    }
 
    plugins = g_list_sort(plugins, plugin_sort);
+   for (temp_list = plugin_names; temp_list; temp_list = temp_list->next) {
+      if (temp_list->data) {
+	 g_free(temp_list->data);
+      }
+   }
+   g_list_free(plugin_names);
 
    return count;
 }
@@ -141,7 +153,6 @@ int load_plugins()
    DIR *dir;
    char path[FILENAME_MAX];
    int count, number;
-   GList *temp_list;
 
    count = 0;
    number = DATEBOOK + 100; /* I just made up this number */
