@@ -1,4 +1,4 @@
-/* $Id: prefs.c,v 1.67 2007-08-07 16:03:06 rikster5 Exp $ */
+/* $Id: prefs.c,v 1.70 2007-10-25 18:53:32 rousseau Exp $ */
 
 /*******************************************************************************
  * prefs.c
@@ -49,7 +49,7 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"todo_hide_completed", INTTYPE, INTTYPE, 0, NULL, 0},
      {"datebook_highlight_days", INTTYPE, INTTYPE, 1, NULL, 0},
      {"port", CHARTYPE, CHARTYPE, 0, NULL, 0},
-     {"rate", CHARTYPE, INTTYPE, 4, NULL, 0},
+     {"rate", CHARTYPE, INTTYPE, 8, NULL, 0},
      {"user", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"user_id", INTTYPE, INTTYPE, 0, NULL, 0},
      {"pc_id", INTTYPE, INTTYPE, 0, NULL, 0},
@@ -132,6 +132,11 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"datebook_anni_years", INTTYPE, INTTYPE, 0, NULL, 0},
      {"keyring_pane", INTTYPE, INTTYPE, 0, NULL, 0},
      {"expense_pane", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"datebook_version", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"address_version", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"todo_version", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"memo_version", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"contacts_photo_filename", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"todo_sort_column", INTTYPE, INTTYPE, 1, NULL, 0},
      {"todo_sort_order", INTTYPE, INTTYPE, 0, NULL, 0},
 };
@@ -924,7 +929,7 @@ int jp_pref_write_rc_file(char *filename, prefType prefs[], int num_prefs)
 	 fprintf(out, "%s %s\n", prefs[i].name, prefs[i].svalue);
       }
    }
-   fclose(out);
+   jp_close_home_file(out);
 
    return EXIT_SUCCESS;
 }
