@@ -1,4 +1,4 @@
-/* $Id: todo_gui.c,v 1.111 2007/06/05 19:39:48 rikster5 Exp $ */
+/* $Id: todo_gui.c,v 1.114 2007-11-25 12:12:33 rousseau Exp $ */
 
 /*******************************************************************************
  * todo_gui.c
@@ -1578,6 +1578,7 @@ static void cb_clist_selection(GtkWidget      *clist,
 
    /*If they have clicked on the checkmark box then do a modify */
    if (column==0) {
+#if 0
       todo->complete = !(todo->complete);
       /* See if we need to mark the due date as the date completed */
       if (todo->complete) {
@@ -1596,6 +1597,8 @@ static void cb_clist_selection(GtkWidget      *clist,
 	 }
       }
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(todo_completed_checkbox), todo->complete);
+#endif
+      gtk_signal_emit_by_name(GTK_OBJECT(todo_completed_checkbox), "clicked");
       gtk_signal_emit_by_name(GTK_OBJECT(apply_record_button), "clicked");
    }
    connect_changed_signals(CONNECT_SIGNALS);
@@ -1688,6 +1691,11 @@ void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    get_pixmaps(clist, PIXMAP_NOTE, &pixmap_note, &mask_note);
    get_pixmaps(clist, PIXMAP_BOX_CHECK, &pixmap_check, &mask_check);
    get_pixmaps(clist, PIXMAP_BOX_CHECKED, &pixmap_checked,&mask_checked);
+#ifdef __APPLE__
+   mask_note = NULL;
+   mask_check = NULL;
+   mask_checked = NULL;
+#endif
 
    entries_shown=0;
    show_priv = show_privates(GET_PRIVATES);
@@ -2117,10 +2125,16 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_clist_set_column_title(GTK_CLIST(clist), TODO_DATE_COLUMN, _("Due"));
    /* Put pretty pictures in the clist column headings */
    get_pixmaps(vbox, PIXMAP_NOTE, &pixmap, &mask);
+#ifdef __APPLE__
+   mask = NULL;
+#endif
    pixmapwid = gtk_pixmap_new(pixmap, mask);
    hack_clist_set_column_title_pixmap(clist, TODO_NOTE_COLUMN, pixmapwid);
 
    get_pixmaps(vbox, PIXMAP_BOX_CHECKED, &pixmap, &mask);
+#ifdef __APPLE__
+   mask = NULL;
+#endif
    pixmapwid = gtk_pixmap_new(pixmap, mask);
    hack_clist_set_column_title_pixmap(clist, TODO_CHECK_COLUMN, pixmapwid);
 
