@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.119 2007/02/16 13:21:34 rousseau Exp $ */
+/* $Id: utils.c,v 1.131 2007-11-06 20:12:45 rikster5 Exp $ */
 
 /*******************************************************************************
  * utils.c
@@ -33,6 +33,7 @@
 #include <unistd.h>
 #include <pi-datebook.h>
 #include <pi-address.h>
+#include <jp-pi-contact.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/file.h>
@@ -60,10 +61,15 @@
 
 /*Stuff for the dialog window */
 extern GtkWidget *glob_dialog;
+extern GtkWidget *glob_date_label;
 int dialog_result;
 
 unsigned int glob_find_id;
 
+/****************************** Prototypes ************************************/
+
+/****************************** Main Code *************************************/
+
 /*
  * Returns usage string that needs to be freed by the caller
  */
@@ -159,6 +165,67 @@ void get_compile_options(char *string, int len)
 	      );
 }
 
+
+void rename_dbnames(char dbname[][32])
+{
+   int i;
+   long datebook_version, address_version, todo_version, memo_version;
+
+   get_pref(PREF_DATEBOOK_VERSION, &datebook_version, NULL);
+   get_pref(PREF_ADDRESS_VERSION, &address_version, NULL);
+   get_pref(PREF_TODO_VERSION, &todo_version, NULL);
+   get_pref(PREF_MEMO_VERSION, &memo_version, NULL);
+   for (i=0; dbname[i] && dbname[i][0]; i++) {
+      if (datebook_version==1) {
+	 if (!strcmp(dbname[i], "DatebookDB.pdb")) {
+	    strcpy(dbname[i], "CalendarDB-PDat.pdb");
+	 }
+	 if (!strcmp(dbname[i], "DatebookDB.pc3")) {
+	    strcpy(dbname[i], "CalendarDB-PDat.pc3");
+	 }
+	 if (!strcmp(dbname[i], "DatebookDB")) {
+	    strcpy(dbname[i], "CalendarDB-Pdat");
+	 }
+      }
+
+      if (address_version==1) {
+	 if (!strcmp(dbname[i], "AddressDB.pdb")) {
+	    strcpy(dbname[i], "ContactsDB-PAdd.pdb");
+	 }
+	 if (!strcmp(dbname[i], "AddressDB.pc3")) {
+	    strcpy(dbname[i], "ContactsDB-PAdd.pc3");
+	 }
+	 if (!strcmp(dbname[i], "AddressDB")) {
+	    strcpy(dbname[i], "ContactsDB-PAdd");
+	 }
+      }
+
+      if (todo_version==1) {
+	 if (!strcmp(dbname[i], "ToDoDB.pdb")) {
+	    strcpy(dbname[i], "TasksDB-PTod.pdb");
+	 }
+	 if (!strcmp(dbname[i], "ToDoDB.pc3")) {
+	    strcpy(dbname[i], "TasksDB-PTod.pc3");
+	 }
+	 if (!strcmp(dbname[i], "ToDoDB")) {
+	    strcpy(dbname[i], "TasksDB-PTod");
+	 }
+      }
+
+      if (memo_version==1) {
+	 if (!strcmp(dbname[i], "MemoDB.pdb")) {
+	    strcpy(dbname[i], "MemosDB-PMem.pdb");
+	 }
+	 if (!strcmp(dbname[i], "MemoDB.pc3")) {
+	    strcpy(dbname[i], "MemosDB-PMem.pc3");
+	 }
+	 if (!strcmp(dbname[i], "MemoDB")) {
+	    strcpy(dbname[i], "MemosDB-PMem");
+	 }
+      }
+   }
+}
+
 int cat_compare(const void *v1, const void *v2)
 {
    struct sorted_cats *s1, *s2;
@@ -181,12 +248,10 @@ int get_timeout_interval()
       return CLOCK_TICK;
    else
       return 60*CLOCK_TICK;
-
 }
 
 gint timeout_date(gpointer data)
 {
-   extern GtkWidget *glob_date_label;
    char str[102];
    char datef[102];
    const char *svalue1, *svalue2;
@@ -397,7 +462,7 @@ const char base64chars[] =
 /* RFC 1341 et seq. */
 void base64_out(FILE *f, char *str)
 {
-   unsigned char *p;
+   char *p;
    int n;
    unsigned int val;
    int pad;
@@ -440,11 +505,11 @@ void ldif_out(FILE *f, char *name, char *fmt, ...)
    int printable = 1;
 
    va_start(ap, fmt);
-   vsnprintf(buf, sizeof(buf), fmt, ap);
+   vsnprintf((char *)buf, sizeof(buf), fmt, ap);
    if (buf[0] == ' ' || buf[0] == ':' || buf[0] == '<')	/* SAFE-INIT-CHAR */ {
       printable = 0;
    }
-   for (p = buf; *p && printable; p++) {
+   for (p = (char *)buf; *p && printable; p++) {
       if (*p < 32 || *p > 126) { /* SAFE-CHAR, excluding all control chars */
 	 printable = 0;
       }
@@ -476,7 +541,7 @@ void ldif_out(FILE *f, char *name, char *fmt, ...)
       }
       *q = '\0';
       fprintf(f, "%s:: ", name);
-      base64_out(f, buf2);
+      base64_out(f, (char *)buf2);
       fprintf(f, "\n");
    }
 }
@@ -1276,7 +1341,7 @@ int dialog_generic(GtkWindow *main_window,
 		  button = gtk_button_new_from_stock(GTK_STOCK_NO);
 	       else
 #endif
-      button = gtk_button_new_with_label(gettext(button_text[i]));
+      button = gtk_button_new_with_label(_(button_text[i]));
       gtk_signal_connect(GTK_OBJECT(button), "clicked",
 			 GTK_SIGNAL_FUNC(cb_dialog_button),
 			 GINT_TO_POINTER(DIALOG_SAID_1 + i));
@@ -1617,7 +1682,7 @@ int jp_close_home_file(FILE *pc_in)
    if (flock(fileno(pc_in), LOCK_UN) < 0)
       jp_logf(JP_LOG_WARN, "unlocking failed: %s\n", strerror(errno));
 
-   fclose(pc_in);
+   return fclose(pc_in);
 }
 
 int rename_file(char *old_filename, char *new_filename)
@@ -1652,26 +1717,29 @@ int check_copy_DBs_to_home()
    char destname[FILENAME_MAX];
    char srcname[FILENAME_MAX];
    struct utimbuf times;
-   char *dbname[]={
+   char dbname_pdb[][32]={
       "DatebookDB.pdb",
 	"AddressDB.pdb",
 	"ToDoDB.pdb",
 	"MemoDB.pdb",
 	"Memo32DB.pdb",
 	"ExpenseDB.pdb",
-	NULL
+	""
    };
 
-   for (i=0; dbname[i]!=NULL; i++) {
-      get_home_file_name(dbname[i], destname, sizeof(destname));
+   /* Convert to new database names if prefs set */
+   rename_dbnames(dbname_pdb);
+   
+   for (i=0; dbname_pdb[i][0]; i++) {
+      get_home_file_name(dbname_pdb[i], destname, sizeof(destname));
       r = stat(destname, &sbuf);
       if (((r)&&(errno==ENOENT)) || (sbuf.st_size==0)) {
 	 /*The file doesn't exist or is zero in size, copy an empty DB file */
-	 if ((strlen(BASE_DIR) + strlen(EPN) + strlen(dbname[i])) > sizeof(srcname)) {
+	 if ((strlen(BASE_DIR) + strlen(EPN) + strlen(dbname_pdb[i])) > sizeof(srcname)) {
 	    jp_logf(JP_LOG_DEBUG, "copy_DB_to_home filename too long\n");
 	    return EXIT_FAILURE;
 	 }
-	 g_snprintf(srcname, sizeof(srcname), "%s/%s/%s/%s", BASE_DIR, "share", EPN, dbname[i]);
+	 g_snprintf(srcname, sizeof(srcname), "%s/%s/%s/%s", BASE_DIR, "share", EPN, dbname_pdb[i]);
 	 in = fopen(srcname, "r");
 	 out = fopen(destname, "w");
 	 if (!in) {
@@ -1738,6 +1806,7 @@ int jp_copy_file(char *src, char *dest)
 
 
 
+#if 0
 /*These next 2 functions were copied from pi-file.c in the pilot-link app */
 /* Exact value of "Jan 1, 1970 0:00:00 GMT" - "Jan 1, 1904 0:00:00 GMT" */
 #define PILOT_TIME_DELTA (unsigned)(2082844800)
@@ -1753,6 +1822,7 @@ unix_time_to_pilot_time (time_t t)
 {
    return (unsigned long)((unsigned long)t + PILOT_TIME_DELTA);
 }
+#endif
 
 unsigned int bytes_to_bin(unsigned char *bytes, unsigned int num_bytes)
 {
@@ -1895,19 +1965,24 @@ int delete_pc_record(AppType app_type, void *VP, int flag)
    MyAppointment *mappt;
    struct Address *addr;
    MyAddress *maddr;
+   struct Contact *cont;
+   MyContact *mcont;
    struct ToDo *todo;
    MyToDo *mtodo;
    struct Memo *memo;
    MyMemo *mmemo;
    char filename[FILENAME_MAX];
-#ifndef PILOT_LINK_0_12
+#ifdef PILOT_LINK_0_12
+   pi_buffer_t *RecordBuffer = NULL;
+#else /* not PILOT_LINK_0_12 */
    unsigned char record[65536];
-#else /* PILOT_LINK_0_12 */
-   pi_buffer_t *RecordBuffer;
-#endif /* PILOT_LINK_0_12 */
+#endif
+//   unsigned char record[65536];
    PCRecType record_type;
    unsigned int unique_id;
-   long ivalue;
+   long ivalue, memo_version;
+
+   jp_logf(JP_LOG_DEBUG, "delete_pc_record(%d, , %d)\n", app_type, flag);
 
    if (VP==NULL) {
       return EXIT_FAILURE;
@@ -1916,6 +1991,7 @@ int delete_pc_record(AppType app_type, void *VP, int flag)
    /* to keep the compiler happy with -Wall*/
    mappt=NULL;
    maddr=NULL;
+   mcont=NULL;
    mtodo=NULL;
    mmemo=NULL;
    switch (app_type) {
@@ -1931,6 +2007,12 @@ int delete_pc_record(AppType app_type, void *VP, int flag)
       unique_id = maddr->unique_id;
       strcpy(filename, "AddressDB.pc3");
       break;
+    case CONTACTS:
+      mcont = (MyContact *) VP;
+      record_type = mcont->rt;
+      unique_id = mcont->unique_id;
+      strcpy(filename, "ContactsDB-PAdd.pc3");
+      break;
     case TODO:
       mtodo = (MyToDo *) VP;
       record_type = mtodo->rt;
@@ -1950,11 +2032,16 @@ int delete_pc_record(AppType app_type, void *VP, int flag)
       mmemo = (MyMemo *) VP;
       record_type = mmemo->rt;
       unique_id = mmemo->unique_id;
+      get_pref(PREF_MEMO_VERSION, &memo_version, NULL);
       get_pref(PREF_MEMO32_MODE, &ivalue, NULL);
       if (ivalue) {
 	 strcpy(filename, "Memo32DB.pc3");
       } else {
-	 strcpy(filename, "MemoDB.pc3");
+         if (memo_version==1) {
+            strcpy(filename, "MemosDB-PMem.pc3");
+         } else {
+            strcpy(filename, "MemoDB.pc3");
+         }
       }
       break;
     default:
@@ -1966,18 +2053,27 @@ int delete_pc_record(AppType app_type, void *VP, int flag)
 	   "It is scheduled to be deleted from the Palm on the next sync.\n"));
       return EXIT_SUCCESS;
    }
+#ifdef PILOT_LINK_0_12
+   RecordBuffer = pi_buffer_new(0);
+#endif
    switch (record_type) {
     case NEW_PC_REC:
     case REPLACEMENT_PALM_REC:
       pc_in=jp_open_home_file(filename, "r+");
       if (pc_in==NULL) {
 	 jp_logf(JP_LOG_WARN, _("Unable to open PC records file\n"));
+#ifdef PILOT_LINK_0_12
+	 pi_buffer_free(RecordBuffer);
+#endif
 	 return EXIT_FAILURE;
       }
       while(!feof(pc_in)) {
 	 read_header(pc_in, &header);
 	 if (feof(pc_in)) {
 	    jp_logf(JP_LOG_WARN, _("Couldn't find record to delete\n"));
+#ifdef PILOT_LINK_0_12
+	    pi_buffer_free(RecordBuffer);
+#endif
 	    jp_close_home_file(pc_in);
 	    return EXIT_FAILURE;
 	 }
@@ -2001,14 +2097,21 @@ int delete_pc_record(AppType app_type, void *VP, int flag)
 	    jp_logf(JP_LOG_WARN, "fseek failed\n");
 	 }
       }
+
+#ifdef PILOT_LINK_0_12
+      pi_buffer_free(RecordBuffer);
+#endif
       jp_close_home_file(pc_in);
       return EXIT_FAILURE;
 
     case PALM_REC:
-      jp_logf(JP_LOG_DEBUG, "Deleting Palm ID %d\n",unique_id);
+      jp_logf(JP_LOG_DEBUG, "Deleting Palm ID %d\n", unique_id);
       pc_in=jp_open_home_file(filename, "a");
       if (pc_in==NULL) {
 	 jp_logf(JP_LOG_WARN, _("Unable to open PC records file\n"));
+#ifdef PILOT_LINK_0_12
+	 pi_buffer_free(RecordBuffer);
+#endif
 	 return EXIT_FAILURE;
       }
       header.unique_id=unique_id;
@@ -2018,107 +2121,144 @@ int delete_pc_record(AppType app_type, void *VP, int flag)
 	 header.rt=DELETED_PALM_REC;
       }
 #ifdef PILOT_LINK_0_12
-      RecordBuffer = pi_buffer_new(0);
-#endif /* PILOT_LINK_0_12 */
       switch (app_type) {
        case DATEBOOK:
 	 appt=&mappt->appt;
 	 /*memset(&appt, 0, sizeof(appt)); */
-#ifndef PILOT_LINK_0_12
+	 if (pack_Appointment(appt, RecordBuffer, datebook_v1) == -1) {
+	    PRINT_FILE_LINE;
+	    jp_logf(JP_LOG_WARN, "pack_Appointment %s\n", _("error"));
+	 }
+	 break;
+       case ADDRESS:
+	 addr=&maddr->addr;
+	 /* memset(&addr, 0, sizeof(addr)); */
+	 if (pack_Address(addr, RecordBuffer, address_v1) == -1) {
+	    PRINT_FILE_LINE;
+	    jp_logf(JP_LOG_WARN, "pack_Address %s\n", _("error"));
+	 }
+	 break;
+       case CONTACTS:
+	 cont=&mcont->cont;
+	 /* memset(&cont, 0, sizeof(cont)); */
+	 //header.rec_len = pack_Contact(cont, record, sizeof(record)-1);
+	 //if (!header.rec_len) {
+	 //   PRINT_FILE_LINE;
+	 //   jp_logf(JP_LOG_WARN, "pack_Contact %s\n", _("error"));
+	 //}
+	 if (pack_Contact(cont, RecordBuffer) == -1) {
+	    PRINT_FILE_LINE;
+	    jp_logf(JP_LOG_WARN, "pack_Contact %s\n", _("error"));
+	 }
+	 break;
+       case TODO:
+	 todo=&mtodo->todo;
+	 /* memset(&todo, 0, sizeof(todo)); */
+	 if (pack_ToDo(todo, RecordBuffer, todo_v1) == -1) {
+	    PRINT_FILE_LINE;
+	    jp_logf(JP_LOG_WARN, "pack_ToDo %s\n", _("error"));
+	 }
+	 break;
+       case MEMO:
+	 memo=&mmemo->memo;
+	 /* memset(&memo, 0, sizeof(memo)); */
+	 if (pack_Memo(memo, RecordBuffer, memo_v1) == -1) {
+	    PRINT_FILE_LINE;
+	    jp_logf(JP_LOG_WARN, "pack_Memo %s\n", _("error"));
+	 }
+	 break;
+       default:
+	 jp_close_home_file(pc_in);
+	 pi_buffer_free(RecordBuffer);
+	 return EXIT_SUCCESS;
+      } /* switch */
+#else /* Not PILOT_LINK_0_12 */
+      switch (app_type) {
+       case DATEBOOK:
+	 appt=&mappt->appt;
+	 /*memset(&appt, 0, sizeof(appt)); */
 	 header.rec_len = pack_Appointment(appt, record, sizeof(record)-1);
 	 if (!header.rec_len) {
 	    PRINT_FILE_LINE;
 	    jp_logf(JP_LOG_WARN, "pack_Appointment %s\n", _("error"));
 	 }
-#else /* PILOT_LINK_0_12 */
-	 if (pack_Appointment(appt, RecordBuffer, datebook_v1) == -1) {
-	    PRINT_FILE_LINE;
-	    jp_logf(JP_LOG_WARN, "pack_Appointment %s\n", _("error"));
-	 }
-#endif /* PILOT_LINK_0_12 */
 	 break;
        case ADDRESS:
 	 addr=&maddr->addr;
 	 /* memset(&addr, 0, sizeof(addr)); */
-#ifndef PILOT_LINK_0_12
 	 header.rec_len = pack_Address(addr, record, sizeof(record)-1);
 	 if (!header.rec_len) {
 	    PRINT_FILE_LINE;
 	    jp_logf(JP_LOG_WARN, "pack_Address %s\n", _("error"));
 	 }
-#else /* PILOT_LINK_0_12 */
-	 if (pack_Address(addr, RecordBuffer, address_v1) == -1) {
+	 break;
+       case CONTACTS:
+	 cont=&mcont->cont;
+	 /* memset(&cont, 0, sizeof(cont)); */
+	 header.rec_len = pack_Contact(cont, record, sizeof(record)-1);
+	 if (!header.rec_len) {
 	    PRINT_FILE_LINE;
-	    jp_logf(JP_LOG_WARN, "pack_Address %s\n", _("error"));
+	    jp_logf(JP_LOG_WARN, "pack_Contact %s\n", _("error"));
 	 }
-#endif /* PILOT_LINK_0_12 */
 	 break;
        case TODO:
 	 todo=&mtodo->todo;
 	 /* memset(&todo, 0, sizeof(todo)); */
-#ifndef PILOT_LINK_0_12
 	 header.rec_len = pack_ToDo(todo, record, sizeof(record)-1);
 	 if (!header.rec_len) {
 	    PRINT_FILE_LINE;
 	    jp_logf(JP_LOG_WARN, "pack_ToDo %s\n", _("error"));
 	 }
-#else /* PILOT_LINK_0_12 */
-	 if (pack_ToDo(todo, RecordBuffer, todo_v1) == -1) {
-	    PRINT_FILE_LINE;
-	    jp_logf(JP_LOG_WARN, "pack_ToDo %s\n", _("error"));
-	 }
-#endif /* PILOT_LINK_0_12 */
 	 break;
        case MEMO:
 	 memo=&mmemo->memo;
 	 /* memset(&memo, 0, sizeof(memo)); */
-#ifndef PILOT_LINK_0_12
 	 header.rec_len = pack_Memo(memo, record, sizeof(record)-1);
 
 	 if (!header.rec_len) {
 	    PRINT_FILE_LINE;
 	    jp_logf(JP_LOG_WARN, "pack_Memo %s\n", _("error"));
 	 }
-#else /* PILOT_LINK_0_12 */
-	 if (pack_Memo(memo, RecordBuffer, memo_v1) == -1) {
-	    PRINT_FILE_LINE;
-	    jp_logf(JP_LOG_WARN, "pack_Memo %s\n", _("error"));
-	 }
-#endif /* PILOT_LINK_0_12 */
 	 break;
        default:
 	 jp_close_home_file(pc_in);
 #ifdef PILOT_LINK_0_12
 	 pi_buffer_free(RecordBuffer);
-#endif
-	 return EXIT_SUCCESS;
-      }
-#ifdef PILOT_LINK_0_12
-      header.rec_len = RecordBuffer->used;
-
 #endif /* PILOT_LINK_0_12 */
+	 return EXIT_SUCCESS;
+      } /* switch */
+#endif
+
+#ifdef PILOT_LINK_0_12
+      header.rec_len = RecordBuffer->used; /* PILOT_LINK_0_12 */
+
       jp_logf(JP_LOG_DEBUG, "writing header to pc file\n");
       write_header(pc_in, &header);
       /* This record be used for making sure that the palm record
        * hasn't changed before we delete it */
       jp_logf(JP_LOG_DEBUG, "writing record to pc file, %d bytes\n", header.rec_len);
-#ifndef PILOT_LINK_0_12
-      fwrite(record, header.rec_len, 1, pc_in);
-#else /* PILOT_LINK_0_12 */
       fwrite(RecordBuffer->data, header.rec_len, 1, pc_in);
-#endif /* PILOT_LINK_0_12 */
       jp_logf(JP_LOG_DEBUG, "record deleted\n");
       jp_close_home_file(pc_in);
-#ifdef PILOT_LINK_0_12
-      pi_buffer_free(RecordBuffer);
+      pi_buffer_free(RecordBuffer); /* PILOT_LINK_0_12 */
+#else /* not PILOT_LINK_0_12 */
+      jp_logf(JP_LOG_DEBUG, "writing header to pc file\n");
+      write_header(pc_in, &header);
+      /* This record be used for making sure that the palm record
+       * hasn't changed before we delete it */
+      jp_logf(JP_LOG_DEBUG, "writing record to pc file, %d bytes\n", header.rec_len);
+      fwrite(record, header.rec_len, 1, pc_in);
+      jp_logf(JP_LOG_DEBUG, "record deleted\n");
+      jp_close_home_file(pc_in);
 #endif
       return EXIT_SUCCESS;
       break;
     default:
       break;
-   }
+   } /* switch (record_type) */
 #ifdef PILOT_LINK_0_12
-   pi_buffer_free(RecordBuffer);
+   if (RecordBuffer)
+	   pi_buffer_free(RecordBuffer);
 #endif
    return EXIT_SUCCESS;
 }
@@ -2143,11 +2283,21 @@ int undelete_pc_record(AppType app_type, void *VP, int flag)
    int found;
    int ret = -1;
    int num;
+   char dbname[][32]={
+      "DatebookDB.pc3",
+	"AddressDB.pc3",
+	"ToDoDB.pc3",
+	"MemoDB.pc3",
+	""
+   };
 
    if (VP==NULL) {
       return EXIT_FAILURE;
    }
 
+   /* Convert to new database names if prefs set */
+   rename_dbnames(dbname);
+   
    /* to keep the compiler happy with -Wall*/
    mappt = NULL;
    maddr = NULL;
@@ -2157,12 +2307,12 @@ int undelete_pc_record(AppType app_type, void *VP, int flag)
     case DATEBOOK:
       mappt = (MyAppointment *) VP;
       unique_id = mappt->unique_id;
-      strcpy(filename, "DatebookDB.pc3");
+      strcpy(filename, dbname[0]);
       break;
     case ADDRESS:
       maddr = (MyAddress *) VP;
       unique_id = maddr->unique_id;
-      strcpy(filename, "AddressDB.pc3");
+      strcpy(filename, dbname[1]);
       break;
     case TODO:
       mtodo = (MyToDo *) VP;
@@ -2172,10 +2322,10 @@ int undelete_pc_record(AppType app_type, void *VP, int flag)
       if (ivalue) {
 	 strcpy(filename, "MaanaDB.pc3");
       } else {
-	 strcpy(filename, "ToDoDB.pc3");
+	 strcpy(filename, dbname[2]);
       }
 #else
-      strcpy(filename, "ToDoDB.pc3");
+      strcpy(filename, dbname[2]);
 #endif
       break;
     case MEMO:
@@ -2185,7 +2335,7 @@ int undelete_pc_record(AppType app_type, void *VP, int flag)
       if (ivalue) {
 	 strcpy(filename, "Memo32DB.pc3");
       } else {
-	 strcpy(filename, "MemoDB.pc3");
+	 strcpy(filename, dbname[3]);
       }
       break;
     default:
@@ -2421,49 +2571,33 @@ int cleanup_pc_files()
    GList *plugin_list, *temp_list;
    struct plugin_s *plugin;
 #endif
+   int i;
+   char dbname[][32]={
+      "DatebookDB",
+	"AddressDB",
+	"ToDoDB",
+	"MemoDB",
+	"Memo32DB",
+	""
+   };
+
+   /* Convert to new database names if prefs set */
+   rename_dbnames(dbname);
 
    fail_flag = 0;
    max_id = max_max_id = 0;
-   jp_logf(JP_LOG_DEBUG, "cleanup_pc_file for DatebookDB\n");
-   ret = cleanup_pc_file("DatebookDB", &max_id);
-   jp_logf(JP_LOG_DEBUG, "max_id was %d\n", max_id);
-   if (ret<0) {
-      fail_flag=1;
-   } else if (max_id > max_max_id) {
-      max_max_id = max_id;
-   }
-   jp_logf(JP_LOG_DEBUG, "cleanup_pc_file for AddressDB\n");
-   ret = cleanup_pc_file("AddressDB", &max_id);
-   jp_logf(JP_LOG_DEBUG, "max_id was %d\n", max_id);
-   if (ret<0) {
-      fail_flag=1;
-   } else if (max_id > max_max_id) {
-      max_max_id = max_id;
-   }
-   jp_logf(JP_LOG_DEBUG, "cleanup_pc_file for ToDoDB\n");
-   ret = cleanup_pc_file("ToDoDB", &max_id);
-   jp_logf(JP_LOG_DEBUG, "max_id was %d\n", max_id);
-   if (ret<0) {
-      fail_flag=1;
-   } else if (max_id > max_max_id) {
-      max_max_id = max_id;
-   }
-   jp_logf(JP_LOG_DEBUG, "cleanup_pc_file for MemoDB\n");
-   ret += cleanup_pc_file("MemoDB", &max_id);
-   jp_logf(JP_LOG_DEBUG, "max_id was %d\n", max_id);
-   if (ret<0) {
-      fail_flag=1;
-   } else if (max_id > max_max_id) {
-      max_max_id = max_id;
-   }
-   jp_logf(JP_LOG_DEBUG, "cleanup_pc_file for Memo32DB\n");
-   ret += cleanup_pc_file("Memo32DB", &max_id);
-   jp_logf(JP_LOG_DEBUG, "max_id was %d\n", max_id);
-   if (ret<0) {
-      fail_flag=1;
-   } else if (max_id > max_max_id) {
-      max_max_id = max_id;
+
+   for (i=0; dbname[i][0]; i++) {
+      jp_logf(JP_LOG_DEBUG, "cleanup_pc_file for %s\n", dbname[i]);
+      ret = cleanup_pc_file(dbname[i], &max_id);
+      jp_logf(JP_LOG_DEBUG, "max_id was %d\n", max_id);
+      if (ret<0) {
+	 fail_flag=1;
+      } else if (max_id > max_max_id) {
+	 max_max_id = max_id;
+      }
    }
+
 #ifdef ENABLE_PLUGINS
    plugin_list = get_plugin_list();
 
@@ -2690,13 +2824,23 @@ char *charset_p2newj(const char *buf, int max_len, int char_set)
 {
    char *newbuf = NULL;
 
-   /* allocate a longer buffer if not done in conversion routine */
-   if (char_set < CHAR_SET_UTF) {
-      newbuf = g_malloc(2*max_len - 1);
-      if (newbuf) {
-	 /* be safe, though string should fit into buf */
-	 g_strlcpy(newbuf, buf, max_len);
+   /* Allocate a longer buffer if not done in conversion routine */
+   /* Only old conversion routines don't assign a buffer */
+   switch (char_set) {
+    case CHAR_SET_JAPANESE:
+    case CHAR_SET_1250:
+    case CHAR_SET_1251:
+    case CHAR_SET_1251_B:
+      if (char_set < CHAR_SET_UTF) {
+         newbuf = g_malloc(2*max_len - 1);
+         if (newbuf) {
+            /* be safe, though string should fit into buf */
+            g_strlcpy(newbuf, buf, max_len);
+         }
       }
+      break;
+    default:
+      break;
    }
 
    switch (char_set) {
@@ -2704,10 +2848,12 @@ char *charset_p2newj(const char *buf, int max_len, int char_set)
     case CHAR_SET_1250 : Win2Lat(newbuf,max_len); break;
     case CHAR_SET_1251 : win1251_to_koi8(newbuf, max_len); break;
     case CHAR_SET_1251_B : koi8_to_win1251(newbuf, max_len); break;
+	case CHAR_SET_LATIN1 : newbuf = g_strdup(buf);
     default:
       newbuf = other_to_UTF(buf, max_len);
       break;
    }
+
    return (newbuf);
 }
 
@@ -2807,8 +2953,6 @@ int pdb_file_delete_record_by_id(char *DB_name, pi_uid_t uid_in)
    int attr;
    int cat;
    pi_uid_t uid;
-   struct stat statb;
-   struct utimbuf times;
 
    jp_logf(JP_LOG_DEBUG, "pdb_file_delete_record_by_id\n");
 
@@ -2817,12 +2961,6 @@ int pdb_file_delete_record_by_id(char *DB_name, pi_uid_t uid_in)
    strcpy(full_local_pdb_file2, full_local_pdb_file);
    strcat(full_local_pdb_file2, "2");
 
-   /* After we are finished, set the create and modify times of new file
-      to the same as the old */
-   stat(full_local_pdb_file, &statb);
-   times.actime = statb.st_atime;
-   times.modtime = statb.st_mtime;
-
    pf1 = pi_file_open(full_local_pdb_file);
    if (!pf1) {
       jp_logf(JP_LOG_WARN, _("Unable to open file: %s\n"), full_local_pdb_file);
@@ -2855,8 +2993,6 @@ int pdb_file_delete_record_by_id(char *DB_name, pi_uid_t uid_in)
       jp_logf(JP_LOG_WARN, "pdb_file_delete_record_by_id(): %s\n", _("rename failed"));
    }
 
-   utime(full_local_pdb_file, &times);
-
    return EXIT_SUCCESS;
 }
 
@@ -2886,8 +3022,6 @@ int pdb_file_modify_record(char *DB_name, void *record_in, int size_in,
    int cat;
    int found;
    pi_uid_t uid;
-   struct stat statb;
-   struct utimbuf times;
 
    jp_logf(JP_LOG_DEBUG, "pi_file_modify_record\n");
 
@@ -2896,12 +3030,6 @@ int pdb_file_modify_record(char *DB_name, void *record_in, int size_in,
    strcpy(full_local_pdb_file2, full_local_pdb_file);
    strcat(full_local_pdb_file2, "2");
 
-   /* After we are finished, set the create and modify times of new file
-      to the same as the old */
-   stat(full_local_pdb_file, &statb);
-   times.actime = statb.st_atime;
-   times.modtime = statb.st_mtime;
-
    pf1 = pi_file_open(full_local_pdb_file);
    if (!pf1) {
       jp_logf(JP_LOG_WARN, _("Unable to open file: %s\n"), full_local_pdb_file);
@@ -2943,8 +3071,6 @@ int pdb_file_modify_record(char *DB_name, void *record_in, int size_in,
       jp_logf(JP_LOG_WARN, "pdb_file_modify_record(): %s\n", _("rename failed"));
    }
 
-   utime(full_local_pdb_file, &times);
-
    return EXIT_SUCCESS;
 }
 
@@ -3018,8 +3144,6 @@ int pdb_file_write_app_block(char *DB_name, void *bufp, int size_in)
    int attr;
    int cat;
    pi_uid_t uid;
-   struct stat statb;
-   struct utimbuf times;
 
    jp_logf(JP_LOG_DEBUG, "pdb_file_write_app_block\n");
 
@@ -3028,12 +3152,6 @@ int pdb_file_write_app_block(char *DB_name, void *bufp, int size_in)
    strcpy(full_local_pdb_file2, full_local_pdb_file);
    strcat(full_local_pdb_file2, "2");
 
-   /* After we are finished, set the create and modify times of new file
-      to the same as the old */
-   stat(full_local_pdb_file, &statb);
-   times.actime = statb.st_atime;
-   times.modtime = statb.st_mtime;
-
    pf1 = pi_file_open(full_local_pdb_file);
    if (!pf1) {
       jp_logf(JP_LOG_WARN, _("Unable to open file: %s\n"), full_local_pdb_file);
@@ -3065,8 +3183,6 @@ int pdb_file_write_app_block(char *DB_name, void *bufp, int size_in)
       jp_logf(JP_LOG_WARN, "pdb_file_write_app_block(): %s\n", _("rename failed"));
    }
 
-   utime(full_local_pdb_file, &times);
-
    return EXIT_SUCCESS;
 }
 
